@@ -10,6 +10,7 @@
 
 var { HttpRequest } = ChromeUtils.import("chrome://tbsync/content/HttpRequest.jsm");
 var { OAuth2 } = ChromeUtils.import("resource:///modules/OAuth2.jsm");
+const { DNS } = ChromeUtils.import("resource:///modules/DNS.jsm");
 
 var network = {
   
@@ -317,20 +318,81 @@ var network = {
   },
  
 
+  checkForRFC6764Request: async function (path, eventLogInfo) {
+      function checkDefaultSecPort (sec) {
+        return sec ? "443" : "80";
+      }
+
+      if (!this.isRFC6764Request(path)) {
+        return path;
+      }
+      
+      let parts = path.toLowerCase().split("6764://");
+      let type = parts[0].endsWith("caldav") ? "caldav" : "carddav";
+
+      // obey preselected security level for DNS lookup
+      // and only use insecure option if specified
+      let scheme = parts[0].startsWith("httpca") ? "http" : "https"; //httpcaldav or httpcarddav = httpca = http      
+      let sec = (scheme == "https");
+      
+      let hostPath = parts[1];
+      while (hostPath.endsWith("/")) { hostPath = hostPath.slice(0,-1); }
+      let host = hostPath.split("/")[0];
+      
+      let result = {};
+      
+      //only perform dns lookup, if the provided path does not contain any path information
+      if (host == hostPath) {
+          let request = "_" + type + (sec ? "s" : "") + "._tcp." + host;
+
+          // get host from SRV record
+          let rv = await DNS.srv(request);                     
+          if (rv && Array.isArray(rv) && rv.length>0 && rv[0].host) {
+              result.secure = sec;
+              result.host = rv[0].host + ((checkDefaultSecPort(sec) == rv[0].port) ? "" : ":" + rv[0].port);
+              TbSync.eventlog.add("info", eventLogInfo, "RFC6764 DNS request succeeded", "SRV record @ " + request + "\n" + JSON.stringify(rv[0]));
+
+              // Now try to get path from TXT
+              rv = await DNS.txt(request);   
+              if (rv && Array.isArray(rv) && rv.length>0 && rv[0].data && rv[0].data.toLowerCase().startsWith("path=")) {
+                  result.path = rv[0].data.substring(5);
+                  TbSync.eventlog.add("info", eventLogInfo, "RFC6764 DNS request succeeded", "TXT record @ " + request + "\n" + JSON.stringify(rv[0]));
+              } else {
+                  result.path = "/.well-known/" + type;
+              }
+
+              result.url = "http" + (result.secure ? "s" : "") + "://" + result.host +  result.path;
+              return result.url;
+          } else {
+              TbSync.eventlog.add("warning", eventLogInfo, "RFC6764 DNS request failed", "SRV record @ " + request);
+          }
+    }
+    
+    // use the provided hostPath and build standard well-known url
+    return scheme + "://" + hostPath + "/.well-known/" + type;
+  },
+
   startsWithScheme: function (url) {
-    return (url.toLowerCase().startsWith("http://") || url.toLowerCase().startsWith("https://"));
+    return (url.toLowerCase().startsWith("http://") || url.toLowerCase().startsWith("https://") || this.isRFC6764Request(url));
+  },
+
+  isRFC6764Request: function (url) {
+    let parts = url.split("6764://");
+    return (parts.length == 2 && parts[0].endsWith("dav"));
   },
 
   sendRequest: async function (requestData, path, method, connectionData, headers = {}, options = {}) {            
-    let url = path;    
+    let url = await this.checkForRFC6764Request(path, connectionData.eventLogInfo);
+    let enforcedPermanentlyRedirectedUrl = (url != path) ? url : null;
+    
     // path could be absolute or relative, we may need to rebuild the full url.
-    if (path.startsWith("http://") || path.startsWith("https://")) {
+    if (url.startsWith("http://") || url.startsWith("https://")) {
       // extract segments from url
-      let uri = Services.io.newURI(path);
+      let uri = Services.io.newURI(url);
       connectionData.https = (uri.scheme == "https");
       connectionData.fqdn = uri.hostPort;
     } else {
-      url = "http" + (connectionData.https ? "s" : "") + "://" + connectionData.fqdn + path;
+      url = "http" + (connectionData.https ? "s" : "") + "://" + connectionData.fqdn + url;
     }
 
     let currentSyncState = connectionData.accountData ? connectionData.accountData.syncData.getSyncState().state : "";
@@ -364,6 +426,9 @@ var network = {
       }
       
       let r = await dav.network.promisifiedHttpRequest(requestData, method, connectionData, headers, options);
+      if (r && enforcedPermanentlyRedirectedUrl && !r.permanentlyRedirectedUrl) {
+        r.permanentlyRedirectedUrl = enforcedPermanentlyRedirectedUrl;
+      }
       
       if (r && r.passwordPrompt && r.passwordPrompt === true) {
         if (i == MAX_RETRIES) {
@@ -411,7 +476,7 @@ var network = {
   },
   
   // Promisified implementation of TbSync's HttpRequest (with XHR interface)
-  promisifiedHttpRequest: function (requestData, method, connectionData, headers, options, aUseStreamLoader) {
+  promisifiedHttpRequest: function (requestData, method, connectionData, headers, options) {
     let responseData = "";
     
     //do not log HEADERS, as it could contain an Authorization header
@@ -422,6 +487,10 @@ var network = {
       options.softfail = [];
     }
     
+    if (!options.hasOwnProperty("responseType")) {
+      options.responseType = "xml";
+    }
+    
     return new Promise(function(resolve, reject) {                  
       let req = new HttpRequest();
 
@@ -439,6 +508,12 @@ var network = {
           }
       }
 
+      if (options.responseType == "base64") {
+        req.responseAsBase64 = true;
+      }
+
+      req.setRequestHeader("User-Agent", dav.sync.prefSettings.getCharPref("clientID.useragent"));
+
       // If this is one of the servers which we use OAuth for, add the bearer token.
       if (connectionData.oauthObj) {
         req.setRequestHeader("Authorization", "Bearer " +  dav.network.getOAuthValue(connectionData.password, "access"));
@@ -539,6 +614,8 @@ var network = {
 
 
           case 200: //returned by DELETE by radicale - watch this !!!
+            return resolve(aResult);
+
           case 204: //is returned by DELETE - no data
           case 201: //is returned by CREATE - no data
             return resolve(null);
