@@ -344,7 +344,7 @@ var tools = {
     //* CARDS OPERATIONS  *
     //* * * * * * * * * * *
 
-    addContact: function(syncData, id, data, etag) {
+    addContact: async function(syncData, id, data, etag) {
         let vCard = data.textContent.trim();
         let vCardData = dav.vCard.parse(vCard);
 
@@ -356,12 +356,12 @@ var tools = {
             card.setProperty("X-DAV-ETAG", etag.textContent);
             card.setProperty("X-DAV-VCARD", vCard);
 
-            dav.tools.setThunderbirdCardFromVCard(syncData, card, vCardData);
+            await dav.tools.setThunderbirdCardFromVCard(syncData, card, vCardData);
             syncData.target.addItem(card);
         }
     },
 
-    modifyContact: function(syncData, id, data, etag) {
+    modifyContact: async function(syncData, id, data, etag) {
         let vCard = data.textContent.trim();
         let vCardData = dav.vCard.parse(vCard);
 
@@ -377,7 +377,7 @@ var tools = {
                 card.setProperty("X-DAV-ETAG", etag.textContent);
                 card.setProperty("X-DAV-VCARD", vCard);
                 
-                dav.tools.setThunderbirdCardFromVCard(syncData, card, vCardData, oCardData);
+                await dav.tools.setThunderbirdCardFromVCard(syncData, card, vCardData, oCardData);
                 syncData.target.modifyItem(card);
             }        
 
@@ -890,7 +890,7 @@ var tools = {
     //MAIN FUNCTIONS FOR UP/DOWN SYNC
 
     //update send from server to client
-    setThunderbirdCardFromVCard: function(syncData, card, vCardData, oCardData = null) {
+    setThunderbirdCardFromVCard: async function(syncData, card, vCardData, oCardData = null) {
         if (TbSync.prefs.getIntPref("log.userdatalevel") > 2) TbSync.dump("JSON from vCard", JSON.stringify(vCardData));
         //if (oCardData) TbSync.dump("JSON from oCard", JSON.stringify(oCardData));
 
@@ -912,9 +912,30 @@ var tools = {
                     case "Photo":
                         {
                             if (newServerValue) {
-                                //set if supported
+                                let type = "";
+                                try {
+                                    type = vCardData[vCardField.item][0].meta.type[0].toLowerCase();
+                                } catch (e) {
+                                    Components.utils.reportError(e);
+                                }
+
+                                // check for inline data or linked data
                                 if (vCardData[vCardField.item][0].meta && vCardData[vCardField.item][0].meta.encoding) {
-                                    card.addPhoto(TbSync.generateUUID() + '.jpg', vCardData["photo"][0].value);
+                                    card.addPhoto(TbSync.generateUUID(), vCardData[vCardField.item][0].value, type || "jpg");
+                                } else  if (vCardData[vCardField.item][0].meta && Array.isArray(vCardData[vCardField.item][0].meta.value) && vCardData[vCardField.item][0].meta.value[0].toString().toLowerCase() == "uri") {
+                                    let connectionData = new dav.network.ConnectionData();
+                                    connectionData.eventLogInfo = syncData.connectionData.eventLogInfo;
+                                    // add credentials, if image is on the account server, go anonymous otherwise
+                                    try {
+                                        if (vCardData[vCardField.item][0].value.split("://").pop().startsWith(syncData.connectionData.fqdn)) {
+                                            connectionData.password = syncData.connectionData.password;
+                                            connectionData.username = syncData.connectionData.username;
+                                        }
+                                        let rv = await dav.network.sendRequest("", vCardData[vCardField.item][0].value , "GET", connectionData, {}, {responseType: "base64"});
+                                        card.addPhoto(TbSync.generateUUID(), rv, type || this.getImageExtension(vCardData[vCardField.item][0].value), vCardData[vCardField.item][0].value);
+                                    } catch(e) {
+                                        TbSync.eventlog.add("warning", syncData.eventLogInfo,"Could not extract externally linked photo from vCard", JSON.stringify(vCardData));
+                                    }
                                 }
                             } else {
                                 //clear
@@ -1057,6 +1078,36 @@ var tools = {
         return {data: newCard, etag: list.getProperty("X-DAV-ETAG"), modified: modified};
     },
 
+    
+    setDefaultMetaButKeepCaseIfPresent: function(defaults, currentObj) {
+        const keys = Object.keys(defaults);
+        for (const key of keys) {
+            let defaultValue = defaults[key];
+
+            // we need to set this value, but do not want to cause a "modified" if it was set like this before, but just with different case
+            // so keep the current case
+            try {
+                let c = currentObj.meta[key][0];
+                if (c.toLowerCase() == defaultValue.toLowerCase()) defaultValue = c;
+            } catch(e) {
+                //Components.utils.reportError(e);                
+            }
+            
+            if (!currentObj.hasOwnProperty("meta")) currentObj.meta = {};
+            currentObj.meta[key]=[defaultValue];
+        }
+    },
+    
+    getImageExtension: function(filename) {
+        // get extension from filename
+	    let extension = "";
+        try {
+            extension = filename.toString().split(".").pop().toUpperCase();
+        } catch (e) {}        
+        return extension;
+    },
+    
+    
     //return the stored vcard of the card (or empty vcard if none stored) and merge local changes
     getVCardFromThunderbirdContactCard: function(syncData, card, generateUID = false) {
         let currentCard = card.getProperty("X-DAV-VCARD").trim();
@@ -1074,11 +1125,19 @@ var tools = {
             switch (property) {
                 case "Photo":
                     {
+                        let extension = this.getImageExtension(card.getProperty("PhotoURI", ""));
+                        let type = (extension == "JPG") ? "JPEG" : extension;
+                        
                         if (card.getProperty("PhotoType", "") == "file") {
                             TbSync.eventlog.add("info", syncData.eventLogInfo, "before photo ("+vCardField.item+")", JSON.stringify(vCardData));
-                            dav.tools.updateValueOfVCard(syncData, property, vCardData, vCardField, card.getPhoto());
+                            dav.tools.updateValueOfVCard(syncData, property, vCardData, vCardField, card.getPhoto());                            
+                            this.setDefaultMetaButKeepCaseIfPresent({encoding : "B", type : type}, vCardData[vCardField.item][0]);
+                            TbSync.eventlog.add("info", syncData.eventLogInfo, "after photo ("+vCardField.item+")", JSON.stringify(vCardData));
+                        } else if (card.getProperty("PhotoType", "") == "web" && card.getProperty("PhotoURI", "")) {
+                            TbSync.eventlog.add("info", syncData.eventLogInfo, "before photo ("+vCardField.item+")", JSON.stringify(vCardData));
+                            dav.tools.updateValueOfVCard(syncData, property, vCardData, vCardField, card.getProperty("PhotoURI", ""));
+                            this.setDefaultMetaButKeepCaseIfPresent({value : "uri", type : type}, vCardData[vCardField.item][0]);
                             TbSync.eventlog.add("info", syncData.eventLogInfo, "after photo ("+vCardField.item+")", JSON.stringify(vCardData));
-                            vCardData[vCardField.item][0].meta = {"encoding": ["b"], "type": ["JPEG"]};
                         }
                     }
                     break;
